/** Rill v0.0.48 https://www.npmjs.com/package/rill License: MIT*/
var Request, fail, getUrl, httpReg, parseDates, urllite;

urllite = require("urllite");

parseDates = require("./util/parseDates");

fail = require("./util/fail");

httpReg = /^https?:\/\//i;


/*
 * Extract a parsed url from a request object.
#
 * @param {Object} req
 */

getUrl = function(req, basePath) {
  var ref, url;
  fail(req.url, "Could not parse an empty url.");
  url = urllite(httpReg.test(req.url) ? req.url : ((((ref = req.socket) != null ? ref.encrypted : void 0)) ? "https" : "http") + "://" + (req.headers.host) + req.url);
  return {
    protocol: url.protocol.slice(0, -1),
    host: url.host,
    hostname: url.hostname,
    port: url.port,
    path: url.pathname.replace(basePath, ""),
    querystring: url.search.slice(1),
    hash: url.hash.slice(1),
    origin: url.origin,
    subdomains: url.host.split('.').reverse().slice(2)
  };
};

Request = (function() {
  function Request(req, basePath) {
    var err, key, ref, ref1, ref2, ref3, ref4, val;
    if (basePath == null) {
      basePath = "";
    }
    fail(req != null, "Could not create Request with an empty object.");
    if (!(this instanceof Request)) {
      return new Request(req, basePath);
    }
    ref = getUrl(req, basePath);
    for (key in ref) {
      val = ref[key];
      this[key] = val;
    }
    this.originalReq = req;
    this.headers = req.headers = (ref1 = req.headers) != null ? ref1 : {};
    this.method = (ref2 = req.method) != null ? ref2 : "GET";
    this.ip = ((ref3 = (ref4 = this.headers['x-forwarded-for']) != null ? ref4.split(',')[0] : void 0) != null ? ref3 : req.connection.remoteAddress);
    try {
      this.query = JSON.parse(this.querystring, parseDates);
    } catch (_error) {
      err = _error;
      this.query = {};
    }
  }


  /*
  	 * Trigger another request that will redirect the browser and send a location change on the server.
  	 * If "back" is the url then it will attempt to redirect the user back a url, otherwise the alt will be used.
  	#
  	 * @param {String} url
  	 * @param {String} alt
   */

  Request.prototype.redirect = function(url, alt) {
    var ref;
    if (url == null) {
      url = "back";
    }
    if (alt == null) {
      alt = "/";
    }
    if (url === "back") {
      url = (ref = this.headers["Referrer"]) != null ? ref : alt;
    }
    this.redirectURL = url;
    return this;
  };

  return Request;

})();

module.exports = Request;
