urllite    = require("urllite")
parseDates = require("./util/parseDates")
fail       = require("./util/fail")
httpReg    = /^https?:\/\//i

###
# Extract a parsed url from a request object.
#
# @param {Object} req
###
getUrl = (req, basePath)->
	fail(req.url, "Could not parse an empty url.")
	url = urllite(
		if httpReg.test(req.url)
			req.url
		else 
			(
				if process.browser then window.location.protocol[0...-1]
				else if req.socket?.encrypted then "https"
				else "http"
			) + "://" + (
				if process.browser then window.location.host
				else req.headers.host
			) + req.url
	)

	protocol:    url.protocol[0...-1]
	host:        url.host
	hostname:    url.hostname
	port:        url.port
	path:        url.pathname.replace(basePath, "")
	querystring: url.search[1...]
	hash:        url.hash[1...]
	origin:      url.origin
	subdomains:  url.host.split('.').reverse().slice(2)


class Request
	constructor: (req, basePath = "")->
		fail(req?, "Could not create Request with an empty object.")
		return new Request(req, basePath) unless this instanceof Request

		@[key] = val for key, val of getUrl(req, basePath)
		@originalReq  = req
		@headers      = req.headers = req.headers ? {}
		@method       = req.method ? "GET"
		@ip           = (
			if process.browser then "127.0.0.1"
			else @headers['x-forwarded-for']?.split(',')[0] ? req.connection.remoteAddress
		)

		try @query = JSON.parse(@querystring, parseDates)
		catch err then @query = {}

	###
	# Trigger another request that will redirect the browser and send a location change on the server.
	# If "back" is the url then it will attempt to redirect the user back a url, otherwise the alt will be used.
	#
	# @param {String} url
	# @param {String} alt
	###
	redirect: (url="back", alt="/")->
		url          = @headers["Referrer"] ? alt if url is "back"
		@redirectURL = url
		return this

module.exports = Request